#include "StdAfx.h"
#include "NotificationProxy.h"
#include "ActorInterface.h"

#define WM_NOTIFICATION_DATA			WM_USER+998

#define IDT_NOTIFICATION				1
	


CDefaultNotificationProxy::CDefaultNotificationProxy(void)
{
#if defined(WIN32)	// __KEH_USE_LKWND__
	m_pWnd = CreateTinyWindow();
	m_pWnd->SetEvents(this);
	m_pWnd->Create(NULL);
#else
	m_pEventDelegate = CreateEventDelegate(NULL);
	m_pEventDelegate->RegisterObserver(this);
#endif	

    m_pMainThreadID = GetCurrentThreadId();
    
	TIMER_START(IDT_NOTIFICATION, 1000, NULL );	// 1ʸ
}


CDefaultNotificationProxy::~CDefaultNotificationProxy(void)
{
	TIMER_KILL(IDT_NOTIFICATION);

	Clear();

#if defined(WIN32) //__KEH_USE_LKWND__
	if(m_pWnd)
	{
		m_pWnd->Delete();
		m_pWnd = NULL;
	}
#else
	m_pEventDelegate->UnRegisterObserver(this);
	m_pEventDelegate->Delete();
	m_pEventDelegate = NULL;
#endif		
}


void CDefaultNotificationProxy::Clear(void)
{
#if defined(WIN32) //__KEH_USE_LKWND__	
	//  ޽  óѴ.
	MSG msg;
	while(PeekMessage(&msg, m_pWnd->GetHWND(), 0, 0, PM_REMOVE))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
#endif	

	std::list<INotifyCommand*>::iterator it = m_listTimerCommand.begin();

	for(; it != m_listTimerCommand.end(); it++)
	{
		delete *it;
	}

	m_listTimerCommand.clear();
}


bool CDefaultNotificationProxy::OnSignalProcess(int nMsg, WPARAM wParam, LPARAM lParam)
{
	switch(nMsg)
	{
		case WM_NOTIFICATION_DATA:
		{
			INotifyCommand *pCommand = reinterpret_cast<INotifyCommand *>(wParam);
			
			DoNotify(pCommand);
		}
		break;
	}
	
	return false;
}


bool CDefaultNotificationProxy::OnTimerProcess(WPARAM wParam, LPARAM lParam)
{
	switch(wParam)
	{
		case IDT_NOTIFICATION:
			DoTimerJob();
			break;
	}
	return false;
}


#if defined(WIN32) //__KEH_USE_LKWND__
BOOL CDefaultNotificationProxy::ITinyWindow_OnWndProc(ITinyWindow * pWindow, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if(uMsg != WM_TIMER)
		return OnSignalProcess(uMsg, wParam, lParam);
	else
		return OnTimerProcess(wParam, lParam);
}
#else
bool CDefaultNotificationProxy::OnGTRunLoopObserver_SourceEvent(GT_EVENT_HANDLE handle, GT_PARAM param1, GT_PARAM param2)
{
	OnSignalProcess(handle, (WPARAM)param1, (LPARAM)param2);
	return true;
}


bool CDefaultNotificationProxy::OnGTRunLoopObserver_TimerEvent(GT_EVENT_HANDLE handle, GT_PARAM param)
{
	OnTimerProcess(handle, (LPARAM)param);
	return true;
}
#endif


void CDefaultNotificationProxy::DoTimerJob(void)
{
	std::list<INotifyCommand*>::iterator it = m_listTimerCommand.begin();

	for(; it != m_listTimerCommand.end(); )
	{
		INotifyCommand *pCommand = (*it);
		if(pCommand->CheckTick())
		{
			it = m_listTimerCommand.erase(it);

			DoNotify(pCommand);
		}
		else
		{
			it++;
		}
	}
}

void CDefaultNotificationProxy::DoNotify(INotifyCommand *pCommand)
{
	if(pCommand)
	{
		pCommand->Execute();
		delete pCommand;
	}
}


void CDefaultNotificationProxy::Notify(INotifyCommand *pCommand)
{
#if defined(WIN32) //__KEH_USE_LKWND__	
	if(m_pWnd == NULL)
		return;
#else
	if(NULL == m_pEventDelegate)
		return;
#endif	

	if(pCommand->GetRemainTickCount() > 0)
	{
		m_listTimerCommand.push_back(pCommand);
	}
	else
	{
        // If this is main thread, don't need to notify on async-mode.. 
        if(m_pMainThreadID == GetCurrentThreadId())
        {
            DoNotify(pCommand);
        }
        else
        {
            SIGNAL_PERFORM(WM_NOTIFICATION_DATA, (WPARAM)pCommand, (LPARAM)NULL);
        }
	}
}

